/*
 * See the NOTICE file distributed with this work for additional
 * information regarding copyright ownership.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.xwiki.attachment.validation;

import java.util.List;

import org.xwiki.component.annotation.Role;
import org.xwiki.stability.Unstable;

/**
 * Configuration values for the attachment validation.
 *
 * @version $Id: 9bdd2dfbf01d783fb757c6d86e7ff58bdb4d1e24 $
 * @since 14.10
 */
@Role
@Unstable
public interface AttachmentValidationConfiguration
{
    /**
     * @return the list of allowed attachment mimetypes. A joker (@code '*') can be used to match any media (e.g.,
     *     "image/png", "text/*")
     */
    List<String> getAllowedMimetypes();

    /**
     * @return the list of blocker attachment mimetype. A joker (@code '*') can be used to match any media (e.g.,
     *     "image/png", "text/*")
     */
    List<String> getBlockerMimetypes();
}
